/* eslint-disable no-undef */
import React from 'react';
import PropTypes from 'prop-types';

class XmlEditor extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
			loading: true,
		};
		this.xmlEditor = React.createRef();
		this.handleError = this.handleError.bind(this);
		this.afterInit = this.afterInit.bind(this);
	}

	componentDidMount() {
		const { devMode, devXmleditorUri, onError, xsdName, xsdProviderUrl, xsdVersion } = this.props;
		if (xsdName !== undefined && xsdName !== '' && xsdName !== null) {
			try {
				const extractor = new Xsd2Json(xsdName, {
					schemaURI: devMode
						? `${devXmleditorUri}/o/xmleditor/`
						: `${xsdProviderUrl}${xsdVersion}/`,
				});
				this.afterInit(extractor);
			} catch (err) {
				onError('Wizualizacja metadanych nie jest możliwa. Struktura niezgodna ze schematem XSD.');
			}
		}
	}

	afterInit(extractor) {
		const {
			baseRestApiUrl,
			blockUiFunc,
			devMode,
			devXmleditorUri,
			errorName,
			id,
			name,
			onAfterStateChange,
			onChange,
			stateField,
			unblockUiFunc,
			viewMode,
			xmlName,
			xmlProviderUrl,
		} = this.props;
		this.setState({ loading: false }, () => {
			const options = {
				externalId: id,
				enforceOccurs: true,
				schema: extractor.getSchema(),
				readOnly: viewMode === 'VIEW',
				enableGUIKeybindings: false,
				onError: this.handleError,
				confirmExitWhenUnsubmitted: false,
				blockUi: blockUiFunc,
				unblockUi: unblockUiFunc,
			};
			if (xmlProviderUrl !== undefined) {
				options.ajaxOptions = {
					xmlRetrievalPath: devMode
						? `${devXmleditorUri}/o/xmleditor/${xmlName}`
						: xmlProviderUrl.startsWith('blob')
						? xmlProviderUrl
						: `${baseRestApiUrl}/${xmlProviderUrl}`,
				};
			}
			$(`#${id}`).xmlEditor(options);
			$(`#${id}`).focusout(function() {
				const xmlString = $(`#${id}`).xmlEditor('getXMLString');
				const result = {};
				result.lastValidXml = xmlString;
				result.name = name;
				result.errorName = errorName;
				onChange('XML', undefined, result, onAfterStateChange, stateField);
			});
		});
	}

	handleError(err) {
		const { onError } = this.props;
		onError(err);
	}

	render() {
		const { error, id, label, validator, validators, xsdName } = this.props;
		const { loading } = this.state;
		let computedValidators;
		if (validators === 'xml_show_error') {
			let computedError = '';
			if (error !== undefined) {
				computedError = error.toString();
				computedError = computedError.substring(6, computedError.indexOf('Below'));
				computedError = computedError.replace(/:/g, ' ');
			}
			computedValidators = `${validators}:${computedError}`;
		} else {
			computedValidators = validators;
		}
		return !loading && xsdName !== undefined && xsdName !== '' && xsdName !== null ? (
			<div className="col-md-12">
				<div className="row">
					<div ref={this.xmlEditor} id={id} className="col-md-12">
						{
							// eslint-disable-next-line max-len
							'<dokument xmlns="http://www.mswia.gov.pl/standardy/ndap" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.mswia.gov.pl/standardy/ndap Metadane-1.7.xsd"> </dokument>'
						}
					</div>
					<div aria-live="assertive">
						{validator ? validator.message(id, label, error, computedValidators) : null}
					</div>
				</div>
			</div>
		) : null;
	}
}

XmlEditor.defaultProps = {
	devMode: false,
	devXmleditorUri: 'http://localhost:8080',
	stateField: 'element',
	validators: 'xml',
	viewMode: 'VIEW',
};

XmlEditor.propTypes = {
	baseRestApiUrl: PropTypes.string.isRequired,
	blockUiFunc: PropTypes.func,
	devMode: PropTypes.bool,
	devXmleditorUri: PropTypes.string,
	error: PropTypes.object,
	errorName: PropTypes.string.isRequired,
	id: PropTypes.string.isRequired,
	label: PropTypes.string.isRequired,
	name: PropTypes.string.isRequired,
	onAfterStateChange: PropTypes.func,
	onChange: PropTypes.func.isRequired,
	onError: PropTypes.func.isRequired,
	stateField: PropTypes.string,
	unblockUiFunc: PropTypes.func,
	validator: PropTypes.object.isRequired,
	validators: PropTypes.string,
	viewMode: PropTypes.string,
	xmlName: PropTypes.string,
	xmlProviderUrl: PropTypes.string.isRequired,
	xsdName: PropTypes.any.isRequired,
	xsdProviderUrl: PropTypes.string.isRequired,
	xsdVersion: PropTypes.any.isRequired,
};
export default XmlEditor;
